pub enum AST {
Show 51 variants
Statement(Box<AST>, Option<LineInfo>),
Omen(bool, Option<LineInfo>),
Arcana(i64, Option<LineInfo>),
Aether(f64, Option<LineInfo>),
Rune(String, Option<LineInfo>),
Abyss(Option<LineInfo>),
Add(Box<AST>, Box<AST>, Option<LineInfo>),
Sub(Box<AST>, Box<AST>, Option<LineInfo>),
Mul(Box<AST>, Box<AST>, Option<LineInfo>),
Div(Box<AST>, Box<AST>, Option<LineInfo>),
Mod(Box<AST>, Box<AST>, Option<LineInfo>),
PowArcana(Box<AST>, Box<AST>, Option<LineInfo>),
PowAether(Box<AST>, Box<AST>, Option<LineInfo>),
Equal(Box<AST>, Box<AST>, Option<LineInfo>),
NotEqual(Box<AST>, Box<AST>, Option<LineInfo>),
LessThan(Box<AST>, Box<AST>, Option<LineInfo>),
LessThanOrEqual(Box<AST>, Box<AST>, Option<LineInfo>),
GreaterThan(Box<AST>, Box<AST>, Option<LineInfo>),
GreaterThanOrEqual(Box<AST>, Box<AST>, Option<LineInfo>),
LogicalAnd(Box<AST>, Box<AST>, Option<LineInfo>),
LogicalOr(Box<AST>, Box<AST>, Option<LineInfo>),
LogicalNot(Box<AST>, Option<LineInfo>),
VarAssign {
name: String,
value: Box<AST>,
var_type: Type,
is_morph: bool,
line_info: Option<LineInfo>,
},
Assignment {
name: String,
value: Box<AST>,
op: AssignmentOp,
line_info: Option<LineInfo>,
},
Var(String, Option<LineInfo>),
Reveal(Box<AST>, Option<LineInfo>),
Oracle {
is_match: bool,
conditionals: Vec<ConditionalAssignment>,
branches: Vec<AST>,
line_info: Option<LineInfo>,
},
OracleBranch {
pattern: Vec<AST>,
guard: Option<Box<AST>>,
body: Box<AST>,
line_info: Option<LineInfo>,
},
OracleDontCareItem(Option<LineInfo>),
OracleScrollPattern {
elements: Vec<AST>,
line_info: Option<LineInfo>,
},
OracleScrollRest {
name: Option<String>,
line_info: Option<LineInfo>,
},
OracleArtifactPattern {
type_name: String,
fields: Vec<(String, AST)>,
line_info: Option<LineInfo>,
},
OracleLexiconPattern {
entries: Vec<(String, AST)>,
line_info: Option<LineInfo>,
},
Block(Vec<AST>, Option<LineInfo>),
Comment(String, Option<LineInfo>),
Orbit {
params: Vec<AST>,
body: Box<AST>,
line_info: Option<LineInfo>,
},
OrbitParam {
name: String,
start: Box<AST>,
end: Box<AST>,
op: String,
line_info: Option<LineInfo>,
},
Resume(Option<String>, Option<LineInfo>),
Eject(Option<String>, Option<LineInfo>),
Engrave {
name: String,
params: Vec<AST>,
return_type: Type,
body: Box<AST>,
method_target: Option<ArtifactMethodTarget>,
line_info: Option<LineInfo>,
},
EngraveParam {
name: String,
param_type: Type,
is_morph: bool,
line_info: Option<LineInfo>,
},
FuncCall {
name: String,
args: Vec<AST>,
line_info: Option<LineInfo>,
},
ListLiteral {
elements: Vec<AST>,
line_info: Option<LineInfo>,
},
MapLiteral {
entries: Vec<(String, AST)>,
line_info: Option<LineInfo>,
},
IndexAccess {
target: Box<AST>,
index: Box<AST>,
line_info: Option<LineInfo>,
},
IndexAssignment {
target: Box<AST>,
index: Box<AST>,
value: Box<AST>,
line_info: Option<LineInfo>,
},
ArtifactDef {
name: String,
fields: Vec<ArtifactField>,
line_info: Option<LineInfo>,
},
ArtifactLiteral {
type_name: String,
fields: Vec<(String, AST)>,
line_info: Option<LineInfo>,
},
FieldAccess {
target: Box<AST>,
field: String,
line_info: Option<LineInfo>,
},
FieldAssignment {
target: Box<AST>,
field: String,
value: Box<AST>,
line_info: Option<LineInfo>,
},
MethodCall {
receiver: Box<AST>,
method: String,
args: Vec<AST>,
line_info: Option<LineInfo>,
},
}Expand description
Represents the abstract syntax tree (AST) for the language.
Variants
Statement(Box<AST>, Option<LineInfo>)
Omen(bool, Option<LineInfo>)
Arcana(i64, Option<LineInfo>)
Aether(f64, Option<LineInfo>)
Rune(String, Option<LineInfo>)
Abyss(Option<LineInfo>)
Add(Box<AST>, Box<AST>, Option<LineInfo>)
Sub(Box<AST>, Box<AST>, Option<LineInfo>)
Mul(Box<AST>, Box<AST>, Option<LineInfo>)
Div(Box<AST>, Box<AST>, Option<LineInfo>)
Mod(Box<AST>, Box<AST>, Option<LineInfo>)
PowArcana(Box<AST>, Box<AST>, Option<LineInfo>)
PowAether(Box<AST>, Box<AST>, Option<LineInfo>)
Equal(Box<AST>, Box<AST>, Option<LineInfo>)
NotEqual(Box<AST>, Box<AST>, Option<LineInfo>)
LessThan(Box<AST>, Box<AST>, Option<LineInfo>)
LessThanOrEqual(Box<AST>, Box<AST>, Option<LineInfo>)
GreaterThan(Box<AST>, Box<AST>, Option<LineInfo>)
GreaterThanOrEqual(Box<AST>, Box<AST>, Option<LineInfo>)
LogicalAnd(Box<AST>, Box<AST>, Option<LineInfo>)
LogicalOr(Box<AST>, Box<AST>, Option<LineInfo>)
LogicalNot(Box<AST>, Option<LineInfo>)
VarAssign
Assignment
Var(String, Option<LineInfo>)
Reveal(Box<AST>, Option<LineInfo>)
Oracle
Fields
conditionals: Vec<ConditionalAssignment>OracleBranch
OracleDontCareItem(Option<LineInfo>)
OracleScrollPattern
Scroll-shape pattern that destructures a scroll scrutinee into
its elements. Each element is one of: OracleDontCareItem,
OracleScrollRest, Var(name) (binding), or any other AST node
(treated as a literal expression to compare against).
OracleScrollRest
Rest segment inside an OracleScrollPattern. name = Some("rest")
for ..rest (binds the unmatched tail to a fresh sub-scroll);
name = None for .. (anonymous, drops the tail).
OracleArtifactPattern
Artifact-shape pattern that matches a TypeName { field, … }
scrutinee. Each (field_name, sub_pattern) entry pulls the named
field out of the artifact and matches it against sub_pattern
(typically Var for binding, a literal for compare, or
OracleDontCareItem to ignore). Fields not listed here are not
matched against — the pattern is non-exhaustive by default, so
users can pick out only the fields they care about.
OracleLexiconPattern
Lexicon-shape pattern that matches a { "key": value, … }
scrutinee. Each (key, sub_pattern) entry pulls the named entry
out of the lexicon and matches it against sub_pattern. Keys not
listed here are not matched against — the pattern is
non-exhaustive by default, mirroring the artifact pattern’s
“pick what you need” ergonomics.
Block(Vec<AST>, Option<LineInfo>)
Comment(String, Option<LineInfo>)
Orbit
OrbitParam
Resume(Option<String>, Option<LineInfo>)
Eject(Option<String>, Option<LineInfo>)
Engrave
Fields
method_target: Option<ArtifactMethodTarget>EngraveParam
FuncCall
ListLiteral
MapLiteral
IndexAccess
IndexAssignment
ArtifactDef
ArtifactLiteral
FieldAccess
FieldAssignment
MethodCall